/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:
   
     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

/* Our wrapper 
   constructors/destructors that call Init/Destroy */

CSObject::CSObject(char *name, char *username, char *hostname, 
                   Server *server, char *realname, long usermodes, time_t ts)
		: Service(name, username, hostname, server, realname, usermodes, ts)
{
	this->Help = new HelpObject("chanserv.help");
}

CSObject::~CSObject(void)
{
	delete this->Help;
}

void CSObject::Process(User *sptr, char *line)
{
	if (!line)
		return;
			
	msgtab.Check(3, sptr, line);

}	

void CSObject::JoinUser(User *sptr, Channel *channel) {

	aRegChan	*reg;
	int		level;

	if(!(reg = Registry->FindChannel(channel->name))) 	// Channel is not registered, bail out.
		return;

	level = Registry->GetOpLevel(reg, sptr->name);

	switch(level) {
		case 0:
			break;
		case 5:
			this->Mode(channel->name, "+o", sptr->name);
			break;
		default:
#ifdef SDEBUG
			DebugLog->Write("Unknown op level %i for %s in channel", level, sptr->name, channel->name);
#endif
			break;
	} 
}

void CSObject::Mode(char *channel, char *modes, char *nick) {
	Connection.Send(":%s MODE %s %s %s", this->name, channel, modes, nick); 
}

void CSObject::CmdRegister(User *sptr, char **args, int argc)
{
	sptr->Message(NickServ, "Registering channel %s", args[1]);
	Registry->AddChannel(args[1], sptr->name);
}

void CSObject::CmdAop(User *sptr, char **args, int argc)
{
	aRegChan	*channel;
	aRegNick	*newop;
	aChanOp		*chanop;
	Link		*f;
	int		i =0;

	channel = Registry->FindChannel(args[1]);
	
	if(!channel) {
		sptr->Message(ChanServ, "Sorry, but this channel is not registered with %s", ChanServ->name);
		return;
	}

	if(!strcasecmp(args[2], "LIST")) {
		sptr->Message(ChanServ, "Listing AOPS for channel %s", channel->name);
		for(f = channel->oplist;f;f = f->next) {
			i++;
			sptr->Message(ChanServ, "%i: %s", i, ((aChanOp *)f->value)->name);
		}
		sptr->Message(ChanServ, "End of AOP list");
		return;
	}
	else if(!strcasecmp(args[2], "ADD")) {
		if(argc > 3) {
			chanop = Registry->FindChannelOp(channel, args[3]);
				if(chanop) {
					chanop->level = 5;
					sptr->Message(ChanServ, "Changed access for %s to AOP.", chanop->name);
					return;
				}
			newop = Registry->FindNick(args[3]);
		}
		else {
			sptr->Message(ChanServ, "Not enough parameters");
			return;
		}

		if(!newop) {
			sptr->Message(ChanServ, "You need a registered nick");
			return;
		}

		sptr->Message(ChanServ, "Added %s as AOP to channel %s", newop->name, channel->name);
		Registry->AddChannelOp(channel, newop, 5);
	}
	else if(!strcasecmp(args[2], "DEL")) {
                if(argc > 3) {
			chanop = Registry->FindChannelOp(channel, args[3]);
                                if(chanop) {
					for(f = channel->oplist;f;f = f->next) {
						if(!strcasecmp(((aChanOp*) f->value)->name,args[3]))
							break;
					}
					sptr->Message(ChanServ, "Removing AOP %s", ((aChanOp*) f->value)->name);
					f->Unlink(&channel->oplist);
                                        return;   
                                } else {
					sptr->Message(ChanServ, "Nickname not found in AOP list.");
					return;
				}
		}
	}
}
