/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:

     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

Mail::Mail(void){}
Mail::~Mail(void){}

int Mail::Send(char *to, char *from, char *subject, char *format, ...) {

	va_list ap;
	char	mail[65535];

	va_start(ap, format);
	vsnprintf(mail, 65535, format, ap);
	va_end(ap);

#ifdef USESMTP
	return this->SendSMTP(to, from, subject, mail);	
#else
	return this->SendPIPE(to, from, subject, mail);
#endif

}

int Mail::SendPIPE(char *to, char *from, char *subject, char *mail) {

	FILE *fp;
  
	fp = popen("/usr/sbin/sendmail -t", "w");
	if(fp) {
		fprintf(fp, "From: %s\n", from);
		fprintf(fp, "To: %s\n", to);
		fprintf(fp, "Subject: %s\n", subject);
		fprintf(fp, "\n%s\n", mail);
		fprintf(fp, ".\n");
		pclose(fp);
#ifdef SDEBUG
		DebugLog->Write("Successfully sent email to %s (%i bytes)", to, strlen(mail));
#endif
		return 1;
	}

#ifdef SDEBUG
        DebugLog->Write("Couldn't pipe to sendmail: %s", strerror(errno));
#endif
	errno = 0;
	return 0;
}

int Mail::SendSMTP(char *to, char *from, char *subject, char *mail){

	Socket 	Mail;
	char	buffer[4096];
	int	status = 0;

	buffer[0] = '\0';

	if(Mail.Connect("127.0.0.1", 25) < 0) {
#ifdef SDEBUG
                DebugLog->Write("Couldn't make a connection to mailserver: %s", strerror(errno));
#endif
		errno = 0;
		return 0;
	}
#ifdef SDEBUG
        else 
                DebugLog->Write("Connected to mailserver.");
#endif
	
	// We got connection, let's be nice and say hello. -GZ

	Mail.Send("HELO %s", MYNAME);

        while(Timers->CTime < (Timers->CTime + 5)) {
		recv(Mail.fd, buffer, 4096, 0);
		if(!match(buffer, "*pleased to meet you*")) {
			status = 1;
			break;
		}
        }
	
	if(!status) {
#ifdef SDEBUG
		DebugLog->Write("[SMTP Step 1] No proper response from mailserver: %s.", buffer);
#endif		
		return 0;
	}
	
	Mail.Send("MAIL FROM: <services@%s>", MYNAME);

	status = 0;

        while(Timers->CTime < (Timers->CTime + 5)) {
		recv(Mail.fd, buffer, 4096, 0);
		if(!match(buffer, "*Sender ok*")) {
			status = 1;
			break;
		} else if(!match(buffer, "55*"))
			break;
        }

	if(!status) {
#ifdef SDEBUG
		DebugLog->Write("[SMTP Step 2] No proper response from mailserver: %s.", buffer);
#endif		
		return 0;
	}

	Mail.Send("RCPT TO: %s", to);
	status = 0;

        while(Timers->CTime < (Timers->CTime + 5)) {
		recv(Mail.fd, buffer, 4096, 0);
		if(!match(buffer, "*Recipient ok*")) {
			status = 1;
			break;
		} else if(!match(buffer, "55*"))
			break;
        }

	if(!status) {
#ifdef SDEBUG
		DebugLog->Write("[SMTP Step 3] No proper response from mailserver: %s.", buffer);
#endif	
		return 0;
	}

	Mail.Send("DATA");
	status = 0;

        while(Timers->CTime < (Timers->CTime + 5)) {
		recv(Mail.fd, buffer, 4096, 0);
		if(!match(buffer, "*on a line by itself*")) {
			status = 1;
			break;
		}
	}

	if(!status) {
#ifdef SDEBUG
		DebugLog->Write("[SMTP Step 4] Can not send data to mailserver.");
#endif
		return 0;
	}
	
	Mail.Send("From: %s", from);
	Mail.Send("To: %s", to);
	Mail.Send("Subject: %s", subject);
	Mail.Send("%s", mail);
	Mail.Send(".");

	status = 0;

        while(Timers->CTime < (Timers->CTime + 5)) {
		recv(Mail.fd, buffer, 4096, 0);
			if(!match(buffer, "*accepted for delivery*")) {
				status = 1;
				break;
			}
		}

	if(!status) {
#ifdef SDEBUG
		DebugLog->Write("[SMTP Step 5] Data sent to mailserver not accepted for delivery.");
#endif
		return 0;
	}

	Mail.Send("QUIT");

	close(Mail.fd);

#ifdef SDEBUG
		DebugLog->Write("Successfully sent email to %s (%i bytes)", to, strlen(mail));
#endif
	return status;
}
