/*
   Sirius IRC Services 2
   Copyright (c) 2000, Sirius IRC Services Team
   All rights reserved.

   Redistribution and use in source and binary forms, with or without modification, are permitted
   provided that the following conditions are met:
   
     * Redistributions of source code must retain the above copyright notice, this list of conditions
       and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright notice, this list of conditions
       and the following disclaimer in the documentation and/or other materials provided with the
       distribution.
     * Neither the name of the Sirius IRC Services Team nor the names of its contributors may be used
       to endorse or promote products derived from this software without specific prior written permission.
     * The source code may not be redistributed for a fee or in closed source
       programs, without approval from the Sirius IRC Services Team, however
       for operating systems where binary distribution is required, if URL
       is passed with the package to get the full source
     * No warranty is given unless stated so by the Sirius IRC Services Team

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS `AS IS'' AND ANY EXPRESS OR
   IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
   BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
   BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "services.h"

Server::Server(char *name, Server *uplink)
{
	this->flags = FLAGS_SERVER;
	MakeString(this->name, name);
	this->uplink = uplink;
}

Server::~Server(void)
{
	if (this->name)
		FreeString(this->name);
	this->flags = 0;
	this->uplink = NULL;
}

/*
 * Shutdown(0)  - Shutdown without synching databases to disk.
 * Shutdown(1)	- Synch databases to disk before shutting down.
 */

void	Server::Shutdown(int mode)
{
	if(!IsMine(this)) {
#ifdef SDEBUG
		DebugLog->Write("Attempted to shutdown remote object: %s", this->name);
#endif
	}
	else {
#ifdef SDEBUG
		DebugLog->Write("Services shut down with code %i...", mode);
#endif
		
		// Save databases to disk if required.

		switch(mode) {
			case 0:
				break;
			case 1:
				if(!ServicesDBR) {	// Services Databases are not ready yet.
#ifdef SDEBUG
					DebugLog->Write("Services shutdown code 1 ignored, databases not ready.");
#endif
					break;
				}

				NickServDB->Replace();
				ChanServDB->Replace();
				break;
			default:
				break;
		}

		ServicesLog->Flush();
#ifdef SDEBUG
		DebugLog->Flush();
#endif
	}
}
